<?php

namespace Drupal\box\Access;

use Drupal\box\Entity\BoxInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\Routing\Route;

/**
 * Provides an access checker for box revisions.
 *
 * @ingroup box_access
 */
class BoxRevisionAccessCheck implements AccessInterface {

  /**
   * The box storage.
   *
   * @var \Drupal\box\BoxStorageInterface
   */
  protected $boxStorage;

  /**
   * The box access control handler.
   *
   * @var \Drupal\Core\Entity\EntityAccessControlHandlerInterface
   */
  protected $boxAccess;

  /**
   * A static cache of access checks.
   *
   * @var array
   */
  protected $access = [];

  /**
   * Constructs a new BoxRevisionAccessCheck.
   *
   * @param \Drupal\Core\Entity\EntityManagerInterface $entity_manager
   *   The entity manager.
   */
  public function __construct(EntityManagerInterface $entity_manager) {
    $this->boxStorage = $entity_manager->getStorage('box');
    $this->boxAccess = $entity_manager->getAccessControlHandler('box');
  }

  /**
   * Checks routing access for the box revision.
   *
   * @param \Symfony\Component\Routing\Route $route
   *   The route to check against.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The currently logged in account.
   * @param int $box_revision
   *   (optional) The box revision ID. If not specified, but $box is, access
   *   is checked for that object's revision.
   * @param \Drupal\box\Entity\BoxInterface $box
   *   (optional) A box object. Used for checking access to a box's default
   *   revision when $box_revision is unspecified. Ignored when $box_revision
   *   is specified. If neither $box_revision nor $box are specified, then
   *   access is denied.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(Route $route, AccountInterface $account, $box_revision = NULL, BoxInterface $box = NULL) {
    if ($box_revision) {
      $box = $this->boxStorage->loadRevision($box_revision);
    }
    $operation = $route->getRequirement('_access_box_revision');
    return AccessResult::allowedIf($box && $this->checkAccess($box, $account, $operation))->cachePerPermissions()->addCacheableDependency($box);
  }

  /**
   * Checks box revision access.
   *
   * @param \Drupal\box\Entity\BoxInterface $box
   *   The box to check.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   A user object representing the user for whom the operation is to be
   *   performed.
   * @param string $op
   *   (optional) The specific operation being checked. Defaults to 'view.'
   *
   * @return bool
   *   TRUE if the operation may be performed, FALSE otherwise.
   */
  public function checkAccess(BoxInterface $box, AccountInterface $account, $op = 'view') {
    $map = [
      'view' => 'view all box revisions',
      'update' => 'revert all box revisions',
      'delete' => 'delete all box revisions',
    ];
    $bundle = $box->bundle();
    $type_map = [
      'view' => "view $bundle box revisions",
      'update' => "revert $bundle box revisions",
      'delete' => "delete $bundle box revisions",
    ];

    if (!$box || !isset($map[$op]) || !isset($type_map[$op])) {
      // If there was no box to check against, or the $op was not one of the
      // supported ones, we return access denied.
      return FALSE;
    }

    // Statically cache access by revision ID, language code, user account ID,
    // and operation.
    $langcode = $box->language()->getId();
    $cid = $box->getRevisionId() . ':' . $langcode . ':' . $account->id() . ':' . $op;

    if (!isset($this->access[$cid])) {
      // Perform basic permission checks first.
      if (!$account->hasPermission($map[$op]) && !$account->hasPermission($type_map[$op]) && !$account->hasPermission('administer box entities')) {
        $this->access[$cid] = FALSE;
        return FALSE;
      }

      // There should be at least two revisions. If the vid of the given box
      // and the vid of the default revision differ, then we already have two
      // different revisions so there is no need for a separate database check.
      // Also, if you try to revert to or delete the default revision, that's
      // not good.
      if ($box->isDefaultRevision() && ($this->boxStorage->countDefaultLanguageRevisions($box) == 1 || $op == 'update' || $op == 'delete')) {
        $this->access[$cid] = FALSE;
      }
      elseif ($account->hasPermission('administer box entities')) {
        $this->access[$cid] = TRUE;
      }
      else {
        // First check the access to the default revision and finally, if the
        // box passed in is not the default revision then access to that, too.
        $this->access[$cid] = $this->boxAccess->access($this->boxStorage->load($box->id()), $op, $account) && ($box->isDefaultRevision() || $this->boxAccess->access($box, $op, $account));
      }
    }

    return $this->access[$cid];
  }

}
